import unittest
import planetmapper
import datetime
import numpy as np
from numpy import array, nan


class TestBody(unittest.TestCase):
    def setUp(self):
        self.obj = planetmapper.Body(
            'jupiter', '2000-01-01', aberration_correction='CN+S'
        )

    def test_setup(self):
        self.assertEqual(self.obj.target_body_id, 599)
        self.assertEqual(self.obj.target_frame, 'IAU_JUPITER')
        self.assertEqual(self.obj.et, -43135.816087188054)
        self.assertEqual(
            self.obj.dtm,
            datetime.datetime(2000, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        )
        self.assertEqual(self.obj.r_eq, 71492.0)
        self.assertEqual(self.obj.r_polar, 66854.0)
        self.assertEqual(self.obj.flattening, 0.0648743915403122)

    def test_subpoint(self):
        self.assertAlmostEqual(self.obj._subpoint_ra, 23.853442709917307, delta=0.0001)
        self.assertAlmostEqual(self.obj._subpoint_dec, 8.586656737972165, delta=0.0001)
        self.assertAlmostEqual(self.obj.subpoint_lat, 3.3106136262222714, delta=0.01)
        self.assertAlmostEqual(self.obj.subpoint_lon, 340.18465394706067, delta=0.01)
        self.assertAlmostEqual(self.obj.subpoint_distance, 690086737.7625527, delta=10)

    def test_target(self):
        self.assertAlmostEqual(self.obj.target_ra, 23.8534426134383, delta=0.001)
        self.assertAlmostEqual(self.obj.target_dec, 8.586656685345513, delta=0.001)
        self.assertAlmostEqual(self.obj.target_distance, 690158217.238101, delta=10)
        self.assertAlmostEqual(
            self.obj.target_light_time, 2302.1200127659686, delta=0.01
        )

    def test_limb(self):
        output = self.obj.limb_radec_by_illumination(npts=5)
        expected = (
            array([23.85126327, nan, nan, 23.8521358, 23.84750193, 23.85126327]),
            array([8.59177249, nan, nan, 8.58113043, 8.58615918, 8.59177249]),
            array([nan, 23.85795285, 23.85835397, nan, nan, nan]),
            array([nan, 8.59051167, 8.5837201, nan, nan, nan]),
        )
        self.assertTrue(np.allclose(output, expected, equal_nan=True))

    def test_illumination(self):
        output = self.obj.illumination_angles_from_lonlat(1, 2)
        expected = (11.007734089805565, 31.824423998504553, 20.81747680519847)
        self.assertTrue(np.allclose(output, expected))
        self.assertTrue(self.obj.test_if_lonlat_illuminated(5, 6))
        self.assertFalse(self.obj.test_if_lonlat_illuminated(5, 180))

    def test_state(self):
        output = self.obj.radial_velocity_from_lonlat(3, 4)
        expected = 21.95717775230934
        self.assertAlmostEqual(output, expected, delta=0.001)


class TestObservation(unittest.TestCase):
    def setUp(self):
        self.obj = planetmapper.BodyXY(
            'saturn', '2001-02-03', nx=4, ny=6, aberration_correction='CN+S'
        )
        self.obj.set_x0(2)
        self.obj.set_y0(4.5)
        self.obj.set_r0(3.14)
        self.obj.set_rotation(42)

    def test_img_backplanes(self):
        # use messed up keys here to implicitly test name normalisation
        backplanes_expected = {
            'lon-GRAPHIC': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 342.27696346, 9.7262005, 6.58454767],
                    [134.21396498, 64.70493702, 49.1954644, 37.56896844],
                    [119.36634602, 87.26731979, 69.63427605, 56.17053313],
                    [124.82567431, 100.27272446, 83.68908719, 70.06921137],
                ]
            ),
            'lat-GRAPHIC': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, -73.22033306, -56.40984183, -37.12224446],
                    [-81.69039665, -65.39459639, -48.36403575, -30.95793469],
                    [-60.91680203, -49.37988882, -35.14505374, -19.09454569],
                    [-42.31261069, -32.71489956, -19.76675925, -4.19370823],
                ]
            ),
            'lon-cEnTriC': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 17.72303654, -9.7262005, -6.58454767],
                    [-134.21396498, -64.70493702, -49.1954644, -37.56896844],
                    [-119.36634602, -87.26731979, -69.63427605, -56.17053313],
                    [-124.82567431, -100.27272446, -83.68908719, -70.06921137],
                ]
            ),
            'lat-CENTRiC         ': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, -69.6662938, -50.77732669, -31.62774667],
                    [-79.82368735, -60.62836949, -42.46795279, -26.01656959],
                    [-55.64431379, -43.4906069, -29.80470605, -15.73106141],
                    [-36.52778378, -27.59457643, -16.29927492, -3.4143613],
                ]
            ),
            'pixel-x': array(
                [
                    [0.0, 1.0, 2.0, 3.0],
                    [0.0, 1.0, 2.0, 3.0],
                    [0.0, 1.0, 2.0, 3.0],
                    [0.0, 1.0, 2.0, 3.0],
                    [0.0, 1.0, 2.0, 3.0],
                    [0.0, 1.0, 2.0, 3.0],
                ]
            ),
            'PIXEL-Y': array(
                [
                    [0.0, 0.0, 0.0, 0.0],
                    [1.0, 1.0, 1.0, 1.0],
                    [2.0, 2.0, 2.0, 2.0],
                    [3.0, 3.0, 3.0, 3.0],
                    [4.0, 4.0, 4.0, 4.0],
                    [5.0, 5.0, 5.0, 5.0],
                ]
            ),
            'ra': array(
                [
                    [52.27742237, 52.27678016, 52.27613794, 52.27549573],
                    [52.27800062, 52.27735841, 52.27671619, 52.27607398],
                    [52.27857888, 52.27793666, 52.27729445, 52.27665223],
                    [52.27915713, 52.27851491, 52.2778727, 52.27723048],
                    [52.27973538, 52.27909317, 52.27845095, 52.27780874],
                    [52.28031364, 52.27967142, 52.27902921, 52.27838699],
                ]
            ),
            'dec': array(
                [
                    [16.81614223, 16.81669574, 16.81724925, 16.81780277],
                    [16.81675697, 16.81731048, 16.81786399, 16.81841751],
                    [16.81737171, 16.81792522, 16.81847873, 16.81903225],
                    [16.81798645, 16.81853996, 16.81909347, 16.81964699],
                    [16.81860119, 16.8191547, 16.81970821, 16.82026173],
                    [16.81921593, 16.81976944, 16.82032295, 16.82087647],
                ]
            ),
            'phase': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 6.11687086, 6.11748104, 6.11765129],
                    [6.11691735, 6.11774093, 6.11818867, 6.11838153],
                    [6.11743282, 6.11811747, 6.11852402, 6.1187095],
                    [6.11758732, 6.11824169, 6.1186287, 6.11879463],
                ]
            ),
            'incidence': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 66.21756434, 53.86676844, 55.38559878],
                    [62.16688538, 41.18023575, 29.28422307, 29.81298738],
                    [49.28838838, 28.20619076, 10.78677825, 14.34554822],
                    [47.85130737, 27.30922095, 13.02629218, 20.1990873],
                ]
            ),
            'emission': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 69.26606098, 58.45069433, 61.18827377],
                    [62.54814228, 43.00535933, 33.51793325, 35.85611047],
                    [47.61296328, 27.43412415, 13.64044678, 19.9055563],
                    [43.79498452, 22.54619372, 7.01282177, 20.1269339],
                ]
            ),
            'AZIMUTH': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 174.26858193, 175.11909984, 177.72008969],
                    [173.10744407, 171.28282608, 171.49971018, 178.22878517],
                    [172.13470328, 166.97229991, 154.0309769, 171.20423761],
                    [173.60805782, 170.84149556, 173.19353731, 162.18650979],
                ]
            ),
            'distance': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 1.32940033e09, 1.32938754e09, 1.32938879e09],
                    [1.32939532e09, 1.32937555e09, 1.32936875e09, 1.32937018e09],
                    [1.32938023e09, 1.32936590e09, 1.32936077e09, 1.32936279e09],
                    [1.32937674e09, 1.32936387e09, 1.32935966e09, 1.32936259e09],
                ]
            ),
            'radial-velocity': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 31.20683773, 33.30596849, 35.40499082],
                    [27.12128602, 29.2205177, 31.31965063, 33.41870303],
                    [25.13487195, 27.23410936, 29.33326274, 31.43234068],
                    [23.14838956, 25.24764906, 27.34682615, 29.4459278],
                ]
            ),
            'doppler': array(
                [
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, 1.0001041, 1.0001111, 1.00011811],
                    [1.00009047, 1.00009747, 1.00010448, 1.00011148],
                    [1.00008384, 1.00009085, 1.00009785, 1.00010485],
                    [1.00007722, 1.00008422, 1.00009122, 1.00009823],
                ]
            ),
            'ring-radius': array(
                [
                    [228957.78029376, 198474.2597485, 170312.78096379, 145823.68508115],
                    [
                        192175.51144006,
                        160987.64747718,
                        132462.60463105,
                        108715.36894146,
                    ],
                    [155908.54014235, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                ]
            ),
            'ring-Lon-GrApHiC': array(
                [
                    [264.99535888, 270.32191921, 277.49094778, 287.27022681],
                    [262.55257487, 268.6402571, 277.49072238, 290.660046],
                    [258.96434659, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                ]
            ),
            '   ring-distance   ': array(
                [
                    [1.32962718e09, 1.32959630e09, 1.32956543e09, 1.32953455e09],
                    [1.32959455e09, 1.32956368e09, 1.32953280e09, 1.32950193e09],
                    [1.32956193e09, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                    [nan, nan, nan, nan],
                ]
            ),
            'km-x': array(
                [
                    [31588.23985958, 45513.88183475, 59439.52380991, 73365.16578507],
                    [18379.45679069, 32305.09876561, 46230.74074101, 60156.38271618],
                    [5170.67372179, 19096.31569695, 33021.95767212, 46947.59964752],
                    [-8038.10934734, 5887.53262806, 19813.17460322, 33738.81657839],
                    [-21246.892416, -7321.25044084, 6604.39153433, 20530.03350949],
                    [-34455.6754849, -20530.03350973, -6604.39153457, 7321.25044084],
                ]
            ),
            'KM-Y': array(
                [
                    [
                        -89082.95502615,
                        -75874.17195731,
                        -62665.38888842,
                        -49456.60581958,
                    ],
                    [
                        -75157.31305099,
                        -61948.52998209,
                        -48739.74691331,
                        -35530.96384442,
                    ],
                    [
                        -61231.67107582,
                        -48022.88800693,
                        -34814.10493809,
                        -21605.32186919,
                    ],
                    [-47306.0291006, -34097.2460317, -20888.46296281, -7679.67989385],
                    [-33380.38712537, -20171.60405654, -6962.82098758, 6245.96208125],
                    [-19454.74515015, -6245.96208119, 6962.82098758, 20171.60405648],
                ]
            ),
        }

        for k, img_expected in backplanes_expected.items():
            with self.subTest(k):
                img_output = self.obj.get_backplane_img(k)
                self.assertTrue(
                    np.allclose(img_output, img_expected, equal_nan=True),
                    msg=f'output for {k}:\n' + repr(img_output) + ',\n',
                )

        with self.subTest('Checking tests exist'):
            keys_to_test = [
                self.obj.standardise_backplane_name(k)
                for k in backplanes_expected.keys()
            ]
            for k in self.obj.backplanes.keys():
                with self.subTest(k):
                    self.assertIn(
                        k,
                        keys_to_test,
                        msg=f'No test for {k}:\n{k!r}: {self.obj.get_backplane_img(k)!r},\n',
                    )

    def test_map_backplanes(self):
        backplanes_expected = {
            'LON-GRAPHIC': array(
                [
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                    [
                        345.0,
                        315.0,
                        285.0,
                        255.0,
                        225.0,
                        195.0,
                        165.0,
                        135.0,
                        105.0,
                        75.0,
                        45.0,
                        15.0,
                    ],
                ]
            ),
            'LAT-GRAPHIC': array(
                [
                    [
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                        -75.0,
                    ],
                    [
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                        -45.0,
                    ],
                    [
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                        15.0,
                    ],
                    [
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                        45.0,
                    ],
                    [
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                        75.0,
                    ],
                ]
            ),
            'LON-CENTRIC': array(
                [
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                    [
                        15.0,
                        45.0,
                        75.0,
                        105.0,
                        135.0,
                        165.0,
                        -165.0,
                        -135.0,
                        -105.0,
                        -75.0,
                        -45.0,
                        -15.0,
                    ],
                ]
            ),
            'LAT-CENTRIC': array(
                [
                    [
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                        -71.77283621,
                    ],
                    [
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                        -39.13427614,
                    ],
                    [
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                        -12.29931265,
                    ],
                    [
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                        12.29931265,
                    ],
                    [
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                        39.13427614,
                    ],
                    [
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                        71.77283621,
                    ],
                ]
            ),
            'RA': array(
                [
                    [
                        52.27802191,
                        52.27814309,
                        52.27843856,
                        52.27882915,
                        52.27921021,
                        52.27947965,
                        52.27956525,
                        52.27944408,
                        52.2791486,
                        52.27875799,
                        52.27837692,
                        52.2781075,
                    ],
                    [
                        52.27676519,
                        52.27708118,
                        nan,
                        nan,
                        nan,
                        nan,
                        52.28078971,
                        52.28047375,
                        52.27970323,
                        52.27868463,
                        52.27769091,
                        52.27698836,
                    ],
                    [
                        52.27611905,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        52.28138728,
                        52.28097369,
                        52.27996505,
                        52.27863165,
                        52.27733084,
                        52.27641117,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        52.28094744,
                        52.27993881,
                        52.27860542,
                        52.27730461,
                        52.27638494,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        52.27962843,
                        52.27860984,
                        52.27761612,
                        52.27691356,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'DEC': array(
                [
                    [
                        16.81792206,
                        16.81778107,
                        16.8176856,
                        16.81766123,
                        16.81771449,
                        16.81783111,
                        16.81797985,
                        16.81812085,
                        16.81821633,
                        16.8182407,
                        16.81818743,
                        16.8180708,
                    ],
                    [
                        16.81849763,
                        16.81812997,
                        nan,
                        nan,
                        nan,
                        nan,
                        16.81864832,
                        16.81901602,
                        16.81926501,
                        16.81932856,
                        16.81918965,
                        16.8188855,
                    ],
                    [
                        16.81941096,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        16.81960821,
                        16.82008956,
                        16.82041552,
                        16.82049872,
                        16.82031686,
                        16.81991871,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        16.82110157,
                        16.82142754,
                        16.82151075,
                        16.82132889,
                        16.82093073,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        16.82215027,
                        16.82221384,
                        16.82207492,
                        16.82177075,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'PIXEL-X': array(
                [
                    [
                        0.92413681,
                        0.70588436,
                        0.37457293,
                        0.01897346,
                        -0.26563522,
                        -0.40299243,
                        -0.35628971,
                        -0.13803728,
                        0.1932843,
                        0.54889397,
                        0.83350267,
                        0.97084972,
                    ],
                    [
                        2.47041661,
                        1.9012797,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        -0.29939201,
                        0.56461346,
                        1.49195282,
                        2.23412088,
                        2.59225422,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        2.48404466,
                        3.45556461,
                        nan,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'PIXEL-Y': array(
                [
                    [
                        2.06317193,
                        2.03033167,
                        2.17334059,
                        2.45388022,
                        2.79678382,
                        3.11017357,
                        3.31007629,
                        3.34292461,
                        3.1999139,
                        2.91936441,
                        2.57645277,
                        2.26306481,
                    ],
                    [
                        1.60718009,
                        1.52154983,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        4.94438874,
                        4.57146681,
                        3.83986654,
                        2.94564413,
                        2.12842581,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        4.85007562,
                        3.67949666,
                        nan,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'KM-X': array(
                [
                    [
                        17205.44774508,
                        14599.92215276,
                        8097.22412062,
                        -560.31413889,
                        -9053.01178217,
                        -15105.29640651,
                        -17095.40276313,
                        -14489.98391414,
                        -7987.12066197,
                        670.68956089,
                        9163.49394774,
                        15215.61335206,
                    ],
                    [
                        44761.48403716,
                        37966.95864654,
                        nan,
                        nan,
                        nan,
                        nan,
                        -44684.14393568,
                        -37890.34440923,
                        -20932.66862273,
                        1644.676651,
                        23791.43300772,
                        39573.12930012,
                    ],
                    [
                        58557.60795856,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        -58529.43458772,
                        -49636.4379909,
                        -27438.20547628,
                        2116.55979729,
                        31107.5222826,
                        51766.12175727,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        -49663.00475073,
                        -27464.93293524,
                        2089.7301836,
                        31080.67643857,
                        51739.34994864,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        -21008.72152138,
                        1568.40127063,
                        23715.12227345,
                        39496.97979808,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'KM-Y': array(
                [
                    [
                        -48145.23866808,
                        -51485.40981525,
                        -53870.13950557,
                        -54660.48132026,
                        -53644.66287905,
                        -51094.8310076,
                        -47694.17111844,
                        -44353.88821024,
                        -41969.04929829,
                        -41178.71002305,
                        -42194.64022517,
                        -44744.5813182,
                    ],
                    [
                        -34070.7434172,
                        -42780.80578774,
                        nan,
                        nan,
                        nan,
                        nan,
                        -32894.59414274,
                        -24183.77178717,
                        -17964.48817009,
                        -15903.46720254,
                        -18552.95146459,
                        -25202.73558062,
                    ],
                    [
                        -12513.82535017,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        -10974.38838118,
                        428.89145511,
                        8570.58594567,
                        11268.66869646,
                        7800.20144814,
                        -904.96604872,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        23917.48825389,
                        32059.50465506,
                        34757.74103153,
                        31289.21795458,
                        22583.80014342,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        49001.96527052,
                        51063.320822,
                        48413.71497506,
                        41763.38569891,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'PHASE': array(
                [
                    [
                        6.11689213,
                        6.11658214,
                        6.11632839,
                        6.11619887,
                        6.11622829,
                        6.11640877,
                        6.11669195,
                        6.11700195,
                        6.11725571,
                        6.11738523,
                        6.1173558,
                        6.11717531,
                    ],
                    [
                        6.11702117,
                        6.11621282,
                        6.11555113,
                        6.11521337,
                        6.11529007,
                        6.11576068,
                        6.11649914,
                        6.11730755,
                        6.11796929,
                        6.11830702,
                        6.11823026,
                        6.1177596,
                    ],
                    [
                        6.11705385,
                        6.1159957,
                        6.11512953,
                        6.11468739,
                        6.11478776,
                        6.1154038,
                        6.11637046,
                        6.11742871,
                        6.11829496,
                        6.11873706,
                        6.11863658,
                        6.11802047,
                    ],
                    [
                        6.11700201,
                        6.11594387,
                        6.11507769,
                        6.11463555,
                        6.11473591,
                        6.11535195,
                        6.1163186,
                        6.11737685,
                        6.1182431,
                        6.11868521,
                        6.11858473,
                        6.11796863,
                    ],
                    [
                        6.11687338,
                        6.11606503,
                        6.11540333,
                        6.11506557,
                        6.11514225,
                        6.11561285,
                        6.11635129,
                        6.1171597,
                        6.11782144,
                        6.11815919,
                        6.11808245,
                        6.1176118,
                    ],
                    [
                        6.1166806,
                        6.1163706,
                        6.11611685,
                        6.11598732,
                        6.11601673,
                        6.1161972,
                        6.11648038,
                        6.11679038,
                        6.11704414,
                        6.11717367,
                        6.11714425,
                        6.11696377,
                    ],
                ]
            ),
            'INCIDENCE': array(
                [
                    [
                        65.34608942,
                        72.71236225,
                        78.20371672,
                        80.57392606,
                        79.27533759,
                        74.60941919,
                        67.63847326,
                        59.97236639,
                        53.61868408,
                        50.66780712,
                        52.30195565,
                        57.84648817,
                    ],
                    [
                        69.9989221,
                        89.18140574,
                        103.83138456,
                        110.49583134,
                        106.80867748,
                        94.15819653,
                        76.0065868,
                        55.27990296,
                        34.62007857,
                        20.92925297,
                        29.18457424,
                        48.97640788,
                    ],
                    [
                        79.90510129,
                        105.80883981,
                        128.207,
                        140.37443262,
                        133.38951058,
                        113.01616185,
                        87.80098821,
                        60.91610612,
                        33.67495552,
                        10.28217217,
                        25.75570379,
                        52.79560999,
                    ],
                    [
                        92.20379326,
                        119.08809896,
                        146.32778971,
                        169.71896134,
                        154.24635965,
                        127.20808254,
                        100.09945128,
                        74.19560773,
                        51.79664615,
                        39.62858395,
                        46.61397837,
                        66.98824707,
                    ],
                    [
                        103.99811814,
                        124.72406394,
                        145.3825486,
                        159.07204949,
                        150.81730574,
                        131.02685108,
                        110.0052791,
                        90.823117,
                        76.1730392,
                        69.50850125,
                        73.19585324,
                        85.84660607,
                    ],
                    [
                        112.36587236,
                        120.03164774,
                        126.38495738,
                        129.33560082,
                        127.70149772,
                        122.15724161,
                        114.65797658,
                        107.29196978,
                        101.80077821,
                        99.430654,
                        100.7292671,
                        105.39512267,
                    ],
                ]
            ),
            'EMISSION': array(
                [
                    [
                        68.22503332,
                        75.33291814,
                        80.25208924,
                        81.84953304,
                        79.74899909,
                        74.44394903,
                        67.1537022,
                        59.60067464,
                        53.84903128,
                        51.86950419,
                        54.46051782,
                        60.5909772,
                    ],
                    [
                        75.27064235,
                        93.86825107,
                        107.25608438,
                        111.83199964,
                        105.8446185,
                        91.52152369,
                        72.4517809,
                        51.44631217,
                        31.52114777,
                        21.92303644,
                        34.02372392,
                        54.38682371,
                    ],
                    [
                        86.01980262,
                        111.71866355,
                        133.08704527,
                        141.80349241,
                        130.60669134,
                        108.3113112,
                        82.30192296,
                        55.00843402,
                        27.55723651,
                        8.37845268,
                        31.32262011,
                        58.83283996,
                    ],
                    [
                        97.70281432,
                        124.99390464,
                        152.4422701,
                        171.62335007,
                        148.6830131,
                        121.17335353,
                        93.98552494,
                        68.28488746,
                        46.9150732,
                        38.1996686,
                        49.39868721,
                        71.69459617,
                    ],
                    [
                        107.55230276,
                        128.55593437,
                        148.47934577,
                        158.07856059,
                        145.9807389,
                        125.61867424,
                        104.73472183,
                        86.13640295,
                        72.74806588,
                        68.17244693,
                        74.16048345,
                        88.48359247,
                    ],
                    [
                        112.85031712,
                        120.40291805,
                        126.15435138,
                        128.13407095,
                        125.54353007,
                        119.41351049,
                        111.77968539,
                        104.67184807,
                        99.75265166,
                        98.15517666,
                        100.25563171,
                        105.56045963,
                    ],
                ]
            ),
            'AZIMUTH': array(
                [
                    [
                        174.12613858,
                        174.25034036,
                        174.13305875,
                        173.94704051,
                        173.79841617,
                        173.65565263,
                        173.39462234,
                        172.93333775,
                        172.41639776,
                        172.30820153,
                        172.86577858,
                        173.63450099,
                    ],
                    [
                        176.74757921,
                        176.06793595,
                        174.73986415,
                        173.60029525,
                        173.70708495,
                        174.47452248,
                        174.82670352,
                        174.05532181,
                        170.31529012,
                        163.41937543,
                        172.83373222,
                        176.35621823,
                    ],
                    [
                        179.82899434,
                        178.33601038,
                        175.13776777,
                        170.52666881,
                        172.66871576,
                        175.82286896,
                        177.31118829,
                        178.12463828,
                        179.834409,
                        143.33932565,
                        174.6603855,
                        178.80071242,
                    ],
                    [
                        177.30962963,
                        178.12312863,
                        179.83140734,
                        143.36278919,
                        174.6630591,
                        178.80214209,
                        179.8275647,
                        178.3339374,
                        175.1338273,
                        170.52068796,
                        172.66478081,
                        175.82064681,
                    ],
                    [
                        174.8256525,
                        174.05534748,
                        170.31806435,
                        163.4272447,
                        172.83637586,
                        176.35645627,
                        176.746808,
                        176.06631324,
                        174.73732071,
                        173.59722022,
                        173.70438808,
                        174.47264187,
                    ],
                    [
                        173.39440561,
                        172.93376691,
                        172.41748292,
                        172.30961146,
                        172.86693652,
                        173.63506083,
                        174.12607457,
                        174.24975571,
                        174.13210641,
                        173.94592798,
                        173.79738708,
                        173.65493539,
                    ],
                ]
            ),
            'DISTANCE': array(
                [
                    [
                        1.32939874e09,
                        1.32940654e09,
                        1.32941211e09,
                        1.32941393e09,
                        1.32941153e09,
                        1.32940555e09,
                        1.32939759e09,
                        1.32938978e09,
                        1.32938422e09,
                        1.32938239e09,
                        1.32938479e09,
                        1.32939077e09,
                    ],
                    [
                        1.32940583e09,
                        1.32942619e09,
                        1.32944069e09,
                        1.32944546e09,
                        1.32943920e09,
                        1.32942360e09,
                        1.32940284e09,
                        1.32938247e09,
                        1.32936797e09,
                        1.32936321e09,
                        1.32936947e09,
                        1.32938507e09,
                    ],
                    [
                        1.32941558e09,
                        1.32944224e09,
                        1.32946122e09,
                        1.32946746e09,
                        1.32945926e09,
                        1.32943884e09,
                        1.32941167e09,
                        1.32938501e09,
                        1.32936602e09,
                        1.32935979e09,
                        1.32936798e09,
                        1.32938841e09,
                    ],
                    [
                        1.32942562e09,
                        1.32945227e09,
                        1.32947126e09,
                        1.32947749e09,
                        1.32946930e09,
                        1.32944888e09,
                        1.32942171e09,
                        1.32939505e09,
                        1.32937606e09,
                        1.32936983e09,
                        1.32937802e09,
                        1.32939844e09,
                    ],
                    [
                        1.32943445e09,
                        1.32945481e09,
                        1.32946932e09,
                        1.32947408e09,
                        1.32946782e09,
                        1.32945222e09,
                        1.32943146e09,
                        1.32941110e09,
                        1.32939659e09,
                        1.32939183e09,
                        1.32939809e09,
                        1.32941369e09,
                    ],
                    [
                        1.32943970e09,
                        1.32944751e09,
                        1.32945307e09,
                        1.32945490e09,
                        1.32945250e09,
                        1.32944652e09,
                        1.32943855e09,
                        1.32943075e09,
                        1.32942518e09,
                        1.32942336e09,
                        1.32942576e09,
                        1.32943174e09,
                    ],
                ]
            ),
            'RADIAL-VELOCITY': array(
                [
                    [
                        30.92227396,
                        30.52927835,
                        29.54959884,
                        28.24572268,
                        26.96700523,
                        26.05607766,
                        25.75703936,
                        26.15003462,
                        27.12976081,
                        28.43368399,
                        29.7124018,
                        30.62328269,
                    ],
                    [
                        35.07416219,
                        34.04936952,
                        31.49479377,
                        28.09481528,
                        24.76033851,
                        22.38483291,
                        21.60492944,
                        22.62971969,
                        25.18461281,
                        28.58491109,
                        31.91939028,
                        34.29457851,
                    ],
                    [
                        37.15524328,
                        35.81378711,
                        32.46986697,
                        28.01928455,
                        23.65436925,
                        20.54469505,
                        19.52369568,
                        20.8651477,
                        24.20961166,
                        28.66074205,
                        33.02566151,
                        36.13479189,
                    ],
                    [
                        37.15521833,
                        35.81379623,
                        32.4699184,
                        28.01937519,
                        23.65448549,
                        20.54481643,
                        19.52380036,
                        20.86521831,
                        24.20963994,
                        28.66073112,
                        33.02562498,
                        36.13475022,
                    ],
                    [
                        35.07413466,
                        34.04941621,
                        31.4949326,
                        28.09503951,
                        24.76061851,
                        22.38512411,
                        21.60518427,
                        22.62990029,
                        25.18470125,
                        28.58491412,
                        31.91933754,
                        34.2945146,
                    ],
                    [
                        30.92235912,
                        30.52940423,
                        29.5497753,
                        28.24594601,
                        26.96725917,
                        26.05633775,
                        25.75727948,
                        26.15023401,
                        27.12990961,
                        28.43378592,
                        29.71247312,
                        30.62334788,
                    ],
                ]
            ),
            'DOPPLER': array(
                [
                    [
                        1.00010315,
                        1.00010184,
                        1.00009857,
                        1.00009422,
                        1.00008996,
                        1.00008692,
                        1.00008592,
                        1.00008723,
                        1.0000905,
                        1.00009485,
                        1.00009911,
                        1.00010215,
                    ],
                    [
                        1.000117,
                        1.00011358,
                        1.00010506,
                        1.00009372,
                        1.0000826,
                        1.00007467,
                        1.00007207,
                        1.00007549,
                        1.00008401,
                        1.00009535,
                        1.00010648,
                        1.0001144,
                    ],
                    [
                        1.00012394,
                        1.00011947,
                        1.00010831,
                        1.00009347,
                        1.00007891,
                        1.00006853,
                        1.00006513,
                        1.0000696,
                        1.00008076,
                        1.00009561,
                        1.00011017,
                        1.00012054,
                    ],
                    [
                        1.00012394,
                        1.00011947,
                        1.00010831,
                        1.00009347,
                        1.00007891,
                        1.00006853,
                        1.00006513,
                        1.0000696,
                        1.00008076,
                        1.00009561,
                        1.00011017,
                        1.00012054,
                    ],
                    [
                        1.000117,
                        1.00011358,
                        1.00010506,
                        1.00009372,
                        1.0000826,
                        1.00007467,
                        1.00007207,
                        1.00007549,
                        1.00008401,
                        1.00009535,
                        1.00010648,
                        1.0001144,
                    ],
                    [
                        1.00010315,
                        1.00010184,
                        1.00009857,
                        1.00009422,
                        1.00008996,
                        1.00008692,
                        1.00008592,
                        1.00008723,
                        1.0000905,
                        1.00009485,
                        1.00009912,
                        1.00010215,
                    ],
                ]
            ),
            'RING-RADIUS': array(
                [
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        78634.12139348,
                        86125.99972505,
                        88448.74857809,
                        85387.87728213,
                        77227.0898578,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        126472.56148867,
                        129915.12929124,
                        125375.4767345,
                        113265.64573721,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'RING-LON-GRAPHIC': array(
                [
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        116.22142624,
                        95.65248497,
                        75.70378724,
                        55.71301125,
                        34.99531113,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        86.61851734,
                        76.36557564,
                        66.15472483,
                        56.65027799,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
            'RING-DISTANCE': array(
                [
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        1.32936256e09,
                        1.32934357e09,
                        1.32933734e09,
                        1.32934553e09,
                        1.32936594e09,
                    ],
                    [
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        nan,
                        1.32930396e09,
                        1.32929919e09,
                        1.32930545e09,
                        1.32932105e09,
                    ],
                    [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
                ]
            ),
        }
        degree_interval = 30
        for k, map_expected in backplanes_expected.items():
            with self.subTest(k):
                img_output = self.obj.get_backplane_map(
                    k, degree_interval=degree_interval
                )
                self.assertTrue(
                    np.allclose(img_output, map_expected, equal_nan=True),
                    msg=f'output for {k}:\n' + repr(img_output) + ',\n',
                )

        with self.subTest('Checking tests exist'):
            keys_to_test = [
                self.obj.standardise_backplane_name(k)
                for k in backplanes_expected.keys()
            ]
            for k in self.obj.backplanes.keys():
                with self.subTest(k):
                    self.assertIn(
                        k,
                        keys_to_test,
                        msg=f'No test for {k}:\n{k!r}: {self.obj.get_backplane_map(k, degree_interval)!r},\n',
                    )


if __name__ == '__main__':
    unittest.main()
